﻿using System.Threading.Tasks;
using FluentAssertions;
using Microsoft.AspNetCore.Mvc;
using NSubstitute;
using Website.Controllers;
using Website.Data;
using Website.Models.Blog;
using Website.Models.Git;
using Website.ViewModels;
using Website.ViewModels.Blog;
using Xunit;

namespace Website.Tests.Controllers {
	public class HomeControllerTests {
		[Fact]
		public async Task Index_ReturnsViewWithModel() {
			var blogApi = Substitute.For<IBlogApi>();
			var gitApi = Substitute.For<IGitApi>();

			blogApi.GetLatestPostAsync().Returns(new BlogPost{Content = ""});

			gitApi.GetRepositories("rob").Returns(new[] {new Repository()});
			gitApi.GetBranches("rob", null).Returns(new[] {new Branch {Commit = new Commit()}});
			gitApi.GetCommit("rob", null, null).Returns(new Commit());

			var expectation = new HomeViewModel {
				BlogPost = new BlogPostViewModel(new BlogPost{Content = ""}, false),
				GitCommit = new GitCommitViewModel(new Repository(), new Branch{Commit = new Commit()}, new Commit())
			};

			var controller = new HomeController(blogApi, gitApi);
			(await controller.Index() as ViewResult).Model.Should().BeEquivalentTo(expectation);
		}
	}
}