﻿using System.Net.Http;
using System.Threading.Tasks;
using Microsoft.Extensions.Caching.Memory;
using Microsoft.Extensions.Logging;
using Robware.Lib.ApiClient;
using Website.Models.Auth;

namespace Website.Data {
	public class AuthenticationProvider : ApiClient, IAuthenticationProvider {
		private readonly ILogger<AuthenticationProvider> _logger;

		public AuthenticationProvider(HttpClient client, IMemoryCache cache, CacheDurations cacheDurations, ILogger<AuthenticationProvider> logger) : base(client, cache, cacheDurations) {
			_logger = logger;
		}

		public async Task<User> Authenticate(LoginRequest request) {
			try {
				return await Post<User>("user/authenticate", request);
			}
			catch (ApiCallException exception) {
				_logger.Log(LogLevel.Error, exception, "Error when attempting to authenticate user");
				return null;
			}
		}

		public async Task<bool> UpdateUserPassword(UpdatePasswordRequest request) {
			try {
				await Patch<object>("user/updatepassword", null, new {request.Username, request.OldPassword, request.NewPassword});
				return true;
			}
			catch (ApiCallException exception) {
				_logger.Log(LogLevel.Error, exception, "Error when attempting to update user password");
				return false;
			}
		}
	}
}